// Bovine MP3-WebIt, version 1.6
//    by Jeff Lawson

#define WIN32_LEAN_AND_MEAN
#define STRICT
#include <windows.h>
#include <wininet.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include "mp3webit.rh"
#include "gen.h"
#include "frontend.h"
#include "autobuff.h"

// macro for eliminating warnings.
#define UNUSED_ARGUMENT(x) x = x

// maximum length of a url.
#define MAX_URL_LENGTH 300

// menu id's for our subclassed menu items
#define IDM_WEBIT 7235              // our private menu item
#define IDM_ABOUT 6843              // our private menu item
#define IDM_INSERTFILE1x 1032       // WinAmp's insert file menu item
#define IDM_OPENLOCATION2x 1039     // WinAmp's open location menu item

// global variables
static const char INISECTION[] = "Bovine MP3-WebIt!";
static const int HISTORYCOUNT = 10;
static char szAppName[] = "Bovine MP3-WebIt! (v1.6)";
static const char PlayListClassName[] = "Winamp PE";      // WinAmp's play list editor class name

// old window procedure pointers.
static WNDPROC OldPlayListProc = NULL;
static WNDPROC OldOpenLocationProc = NULL;

// function prototypes for the message processing callbacks.
static LRESULT CALLBACK PlayListProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
static LRESULT CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
static LRESULT CALLBACK OpenLocationProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);

// function prototypes for the winamp plugin callbacks.
static int init(void);
static void config(void);
static void quit(void);


// plugin interface structure
static winampGeneralPurposePlugin plugin =
{
  GPPHDR_VER,
  szAppName,
  init,
  config,
  quit,
};


//////////////////////////////////////////////////////////////////////////////

extern "C" __declspec( dllexport )
winampGeneralPurposePlugin * winampGetGeneralPurposePlugin()
{
  return &plugin;
}

//////////////////////////////////////////////////////////////////////////////

static void IconMessageBoxOk(HWND parent, const char *message)
{
  MSGBOXPARAMS msgbox;
  msgbox.cbSize = sizeof(MSGBOXPARAMS);
  msgbox.hwndOwner = parent;
  msgbox.hInstance = plugin.hDllInstance;
  msgbox.lpszText = message;
  msgbox.lpszCaption = szAppName;
  msgbox.dwStyle = MB_USERICON | MB_OK;
  msgbox.lpszIcon = MAKEINTRESOURCE(IDI_ICON1);
  msgbox.dwContextHelpId = 0;
  msgbox.lpfnMsgBoxCallback = NULL;
  msgbox.dwLanguageId = MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT);
  MessageBoxIndirect(&msgbox);
}

//////////////////////////////////////////////////////////////////////////////

#ifndef WINAMP1         // Winamp 2.x window subclassing
static BOOL CALLBACK EnumWindowsInitProc(
    HWND  hwnd, // handle of parent window
    LPARAM  lParam  // application-defined value
   )
{
  UNUSED_ARGUMENT(lParam);
  char szClassname[80];
  if (GetClassName(hwnd, szClassname, sizeof(szClassname)) &&
      strcmpi(szClassname, PlayListClassName) == 0)
  {
    DWORD windowPid;
    GetWindowThreadProcessId(hwnd, &windowPid);
    if (windowPid == GetCurrentProcessId())
    {
      // we found a playlist window belonging to this process.
      OldPlayListProc = (WNDPROC)SetWindowLong(hwnd, GWL_WNDPROC, (LONG) PlayListProc);
      return FALSE;     // halt enumeration
    }
  }
  return TRUE;      // continue enumeration
}


static long initthread(long)
{
  Sleep(2000);      // allow winamp time to finish starting up.
  EnumWindows((WNDENUMPROC) EnumWindowsInitProc, NULL);
  return 0;
}
#endif          // Winamp 2.x - window subclassing

//////////////////////////////////////////////////////////////////////////////

// WINAMP Callback interface function.
static void config(void)
{
  DialogBox(plugin.hDllInstance,MAKEINTRESOURCE(IDD_DIALOG1),
    plugin.hwndParent,(DLGPROC)ConfigProc);
}

// WINAMP Callback interface function.
static void quit(void)
{
  // nothing
}


// WINAMP Callback interface function.
static int init(void)
{
#ifdef WINAMP1
  // Winamp 1.x - class subclassing
  HWND plwnd = CreateWindow(PlayListClassName, "Dummy Play List",
      WS_SYSMENU | WS_CAPTION | WS_VISIBLE, 0, 0, 100, 100,
      NULL, NULL, NULL, NULL);
  if (plwnd)
  {
    OldPlayListProc = (WNDPROC)SetClassLong(plwnd, GCL_WNDPROC, (LONG) PlayListProc);
    DestroyWindow(plwnd);
  }
  else
  {
    OldPlayListProc = NULL;
    IconMessageBoxOk(NULL, "Unable to subclass play list editor class!");
  }
#else
  // Winamp 2.x - Window subclassing
  DWORD dwThreadId = 0;
  HANDLE hThread = CreateThread(NULL, 0,
      (LPTHREAD_START_ROUTINE)initthread, NULL, 0, &dwThreadId);
  if (!hThread) {
    IconMessageBoxOk(NULL, "Could not create initialization thread.");
  } else {
    CloseHandle(hThread);
  }
#endif
  return 0;
}

//////////////////////////////////////////////////////////////////////////////

void AboutBox(HWND parent)
{
  IconMessageBoxOk(parent,
    "The Bovine MP3-WebIt plugin is a freeware project written by Jeff Lawson.\n"
    "\thttp://www.bovine.net/\n\n"
    "To find out about cows and how much they like "
    "to do distributed computing, please visit our site:\n"
    "\thttp://www.distributed.net/\n");
}

//////////////////////////////////////////////////////////////////////////////

// returns false on error
bool FetchHttpDocument(LPCTSTR hostname, INTERNET_PORT port,
           LPCTSTR objectname, LPTSTR *output)
{
  bool success = false;

  HINTERNET hInternet = InternetOpen(
      "MP3-WebIt", INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
  if (hInternet != NULL)
  {
    HINTERNET hConnection = InternetConnect(
        hInternet, hostname, port, NULL, NULL, INTERNET_SERVICE_HTTP, 0, 0);
    if (hConnection != NULL)
    {
      HINTERNET hRequest = HttpOpenRequest(
          hConnection, "GET", objectname, NULL, NULL, NULL, 0, NULL);
      if (hRequest != NULL)
      {
        if (HttpSendRequest(hRequest, NULL, 0, NULL, 0))
        {
          char *lpszData = NULL;
          DWORD dwSizeSum = 0;
          for (;;)
          {
            // The call to InternetQueryDataAvailable determines the amount
            // of data available to download.
            DWORD dwSize;
            if (!InternetQueryDataAvailable(hRequest,&dwSize,0,0))
              break;
            else if (lpszData == NULL || dwSize != 0)
            {
              // Allocate a buffer of the size returned.
              char *lpszNewBuffer = new char[dwSizeSum + dwSize + 1];
              if (lpszData != NULL) {
                memmove(lpszNewBuffer, lpszData, dwSizeSum);
                delete [] lpszData;
              }
              lpszData = lpszNewBuffer;
            }


            // Read the data from the HINTERNET handle.
            DWORD dwDownloaded;       // indicates when we are done.
            if(!InternetReadFile(hRequest,(LPVOID)lpszData,dwSize,&dwDownloaded))
              break;
            else
            {
              // Add a null terminator to the end of the data buffer.
              dwSizeSum += dwDownloaded;
              lpszData[dwSizeSum] = '\0';

              // Check the size of the remaining data.  If it is zero, break.
              if (dwDownloaded == 0) { success = true; break; }
            }
          }
          if (!success && lpszData != NULL)
            delete [] lpszData;
          else
            *output = lpszData;
        }
        InternetCloseHandle(hRequest);
      }
      InternetCloseHandle(hConnection);
    }
    InternetCloseHandle(hInternet);
  }
  return success;
}

//////////////////////////////////////////////////////////////////////////////

// returns true if a complete line was found
bool RemoveTag(AutoBuffer &buffer, AutoBuffer &line)
{
  line.Clear();

  // seek forward to the first non-blank
  if (*buffer.GetHead() &&
      isspace((unsigned char)*buffer.GetHead()))
    for (int eos = 1; ; eos++)
      if (!buffer.GetHead()[eos] ||
        !isspace((unsigned char)buffer.GetHead()[eos]))
      { buffer.RemoveHead(eos); break; }

  // make sure we have something to parse
  if (buffer.GetLength() == 0) return false;

  // determine the ending character that we are looking for
  unsigned int pos;
  if (buffer.GetHead()[0] == '<')
    {
      // copy inclusive to next '>'
      for (pos = 1; pos < buffer.GetLength(); pos++)
        {
          char ch = buffer.GetHead()[pos];
          if (ch == '>') {pos++; break;}
        }
    }
  else
    {
      // copy exclusive to next '<'
      for (pos = 1; pos < buffer.GetLength(); pos++)
        {
          char ch = buffer.GetHead()[pos];
          if (ch == '<') break;
        }
    }

  // transfer the data
  line.Reserve(pos + 1);
  memmove(line.GetHead(), buffer.GetHead(), pos);
  line.GetHead()[pos] = 0;
  line.MarkUsed(pos + 1);
  buffer.RemoveHead(pos);
  return true;
}

//////////////////////////////////////////////////////////////////////////////

static bool __strniequal(const char *str1, const char *str2, int targetlen)
{
  assert(str1 != NULL && str2 != NULL);
  while (targetlen-- > 0)
  {
    if (toupper(*str1) != toupper(*str2)) return false;    // unequal
    if (!*str1 && !*str2) return true;      // equal
    str1++; str2++;
  }
  return true;   // equal
}

static int __substri(const char *source, const char *pattern)
{
  assert(source != NULL && pattern != NULL);
  const char *sourceorig = source;
  while (*source)
  {
    for (const char *i = source, *j = pattern; *i; i++, j++)
    {
      if (!*j) return (source - sourceorig);   // match found.
      if (toupper(*i) != toupper(*j)) break;
    }
    source++;
  }
  return -1;    // no match found.
}

//////////////////////////////////////////////////////////////////////////////

BOOL WinAmp_AddToPlaylist(HWND hwndWinamp, const char *szFilename)
{
#ifdef WINAMP1
  int length = strlen(szFilename);
  for (int i = 0; i < length; i++) {
    PostMessage(hwndWinamp, WM_WA_IPC, (WPARAM)szFilename[i], IPC_PLAYFILE);
  }
  PostMessage(hwndWinamp, WM_WA_IPC, 0, IPC_PLAYFILE);
  //Sleep(10);
#else
  COPYDATASTRUCT cds;
  cds.dwData = 0x64;
  cds.lpData = (char*) szFilename;
  cds.cbData = strlen(szFilename) + 1;
  SendMessage(hwndWinamp, WM_COPYDATA, 0, (LPARAM) &cds);
#endif
  return TRUE;
}

//////////////////////////////////////////////////////////////////////////////

bool RetrievePlayList(HWND hwnd_winamp, LPCTSTR user_url)
{
  char *buffer;
  URL_COMPONENTS urlcomponents;
  TCHAR szFullUrl[MAX_URL_LENGTH];
  TCHAR szHostname[MAX_URL_LENGTH];

  // fill the structures with buffers.
  ZeroMemory(&urlcomponents, sizeof(URL_COMPONENTS));
  urlcomponents.dwStructSize = sizeof(URL_COMPONENTS);
  urlcomponents.dwHostNameLength = MAX_URL_LENGTH;
  urlcomponents.lpszHostName = szHostname;
  urlcomponents.dwUrlPathLength = 1;
  urlcomponents.lpszUrlPath = NULL;

  // parse the URL into its components.
  DWORD dwUrlLength = MAX_URL_LENGTH;
  if (!InternetCanonicalizeUrl(user_url, szFullUrl, &dwUrlLength, ICU_BROWSER_MODE) ||
      !InternetCrackUrl(szFullUrl, 0, 0, &urlcomponents))
  {
    IconMessageBoxOk(hwnd_winamp, "The URL you entered appears to be invalid.  "
                     "Verify that you have entered the URL correctly.");
    return false;
  }
  if (urlcomponents.nScheme != INTERNET_SCHEME_HTTP)
  {
    IconMessageBoxOk(hwnd_winamp, "The URL you entered does not appear to "
                     "have a supported scheme.  Only HTTP is allowed.");
    return false;
  }


  // retrieve the data from the remote host
  if (!FetchHttpDocument(urlcomponents.lpszHostName, urlcomponents.nPort,
                         urlcomponents.lpszUrlPath, &buffer))
  {
    IconMessageBoxOk(hwnd_winamp, "Error retreiving document from "
      "remote host.  Verify that you have entered the URL correctly.");
    return false;
  }


  // start parsing out chunks
  AutoBuffer htmlbuff(buffer);
  AutoBuffer chunk;
  int songcount = 0;
  while (RemoveTag(htmlbuff, chunk))
  {
    if (__strniequal(chunk, "<a ", 2))
    {
      int pos = __substri(chunk, " href=\"");
      if (pos > 0)
      {
        chunk.RemoveHead(pos + 7);
        pos = __substri(chunk, "\"");
        if (pos >= 0)
        {
          chunk.RemoveTail(chunk.GetLength() - pos);
          *chunk.GetTail() = '\0';

          // generate the full url from the relative path
          char szNewFile[MAX_URL_LENGTH];
          DWORD dwNewUrlLength = MAX_URL_LENGTH;
          if (InternetCombineUrl(szFullUrl, chunk, szNewFile,
                                 &dwNewUrlLength, ICU_BROWSER_MODE))
          {
            // fill the structures with buffers.
            ZeroMemory(&urlcomponents, sizeof(URL_COMPONENTS));
            urlcomponents.dwStructSize = sizeof(URL_COMPONENTS);
            urlcomponents.dwUrlPathLength = 1;
            urlcomponents.lpszUrlPath = NULL;

            if (InternetCrackUrl(szNewFile, 0, 0, &urlcomponents) &&
                urlcomponents.nScheme == INTERNET_SCHEME_HTTP &&
                lstrlen(szNewFile) > 4 &&
                lstrcmpi(strchr(szNewFile, 0) - 4, ".mp3") == 0)
            {
              WinAmp_AddToPlaylist(hwnd_winamp, szNewFile);
              songcount++;
            }
          }
        }
      }
    }
  }

  if (songcount == 0)
  {
    IconMessageBoxOk(hwnd_winamp, "No linked MP3's could be found "
       "on the the web page that you specified.  Verify that you "
       "have typed the URL correctly.");
    return false;
  }
  else
  {
    char buffer[200];
    wsprintf(buffer, "A total of %i linked MP3's were found on the "
      "specified web page and added to your play list.", songcount);
    IconMessageBoxOk(hwnd_winamp, buffer);
    return true;
  }
}

//////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK PlayListProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static int urldialogtrigger = 0;

#ifdef WINAMP1
  if (uMsg == WM_INITDIALOG)
  {
    // Winamp 1.x - let the original window proc handle it first
    LRESULT retcode = CallWindowProc(OldPlayListProc, hwndDlg, uMsg, wParam, lParam);
    HMENU menu = GetMenu(hwndDlg);
    if (menu)     // WinAmp 1.x has a menu, but 2.x doesn't
    {
      // create a popup menu to contain our new items
      HMENU popup = CreatePopupMenu();
      AppendMenu(popup, MF_ENABLED, IDM_WEBIT, "Add MP3's linked from web page");
      AppendMenu(popup, MF_SEPARATOR, 0, NULL);
      AppendMenu(popup, MF_ENABLED, IDM_ABOUT, "About Bovine MP3-WebIt");

      // add our popup to the end of the menu
      AppendMenu(menu, MF_ENABLED | MF_POPUP, (UINT) popup, "WebIt");
    }
    return retcode;
  }
  else if (uMsg == WM_COMMAND && LOWORD(wParam) == IDM_WEBIT)
  {
    // Winamp 1.x - catch requests to open our dialog
    DialogBox(plugin.hDllInstance, MAKEINTRESOURCE(IDD_DIALOG1),
      hwndDlg, (DLGPROC)ConfigProc);
    return FALSE;
  }
  else if (uMsg == WM_COMMAND && LOWORD(wParam) == IDM_ABOUT)
  {
    // Winamp 1.x - catch requests to display our about dialog
    AboutBox(hwndDlg);
    return FALSE;
  }
  else if (uMsg == WM_INITMENUPOPUP)
  {
    // Winamp 1.x
    BOOL result = OldPlayListProc(hwndDlg, uMsg, wParam, lParam);

    // Add an item to the "Add" button's popup menu.   The "Add"
    // menu has "Insert File" as the first item, so look for that.
    HMENU menu = (HMENU) LOWORD(wParam);
    if (menu)
    {
      int id = GetMenuItemID(menu, 0);
      if (id == IDM_INSERTFILE1x)
        AppendMenu(menu, MF_ENABLED, IDM_WEBIT, "&Weblinks\tMoo!");
    }
    return result;
  }
#else     // Winamp 2.x
  if (uMsg == WM_COMMAND &&
      HIWORD(wParam) == 0 && LOWORD(wParam) == IDM_OPENLOCATION2x)
  {
    // Winamp 2.x - the "Add/URL" menu item was picked.
    urldialogtrigger = 100;
    // allow drop-through to old handler
  }
  else if (uMsg == WM_ACTIVATE &&
      LOWORD(wParam) == WA_INACTIVE &&
      HIWORD(wParam) == 0 && urldialogtrigger > 0)
  {
    // Winamp 2.x - the "Add/URL" dialog box is being opened.
    urldialogtrigger = 0;
    HWND hwndDialog = (HWND) lParam;
    if (IsWindow(hwndDialog))
    {
      RECT buttonRect;
      HWND buttonHwnd = GetDlgItem(hwndDialog, IDOK);
      if (buttonHwnd)
      {
        // Compute the position of the OK button relative to the dialog.
        POINT clientPos = {0, 0};
        ClientToScreen(hwndDialog, &clientPos);
        GetWindowRect(buttonHwnd, &buttonRect);
        OffsetRect(&buttonRect, -clientPos.x, -clientPos.y);

        // Enlarge the height of the dialog.
        RECT dialogRect;
        GetWindowRect(hwndDialog, &dialogRect);
        SetWindowPos(hwndDialog, 0, 0, 0,
                     (dialogRect.right - dialogRect.left),
                     (dialogRect.bottom - dialogRect.top) +
                      (buttonRect.bottom - buttonRect.top) + 4,
                     SWP_NOREPOSITION | SWP_NOMOVE);

        // Now compute the position of the new button (under the OK button),
        // and the width of the entire dialog box, minus the border gap.
        OffsetRect(&buttonRect, 0, (buttonRect.bottom - buttonRect.top) + 4);
        buttonRect.right = dialogRect.right - dialogRect.left - buttonRect.left;


        // Add the new button into the dialog.
        CreateWindowEx(WS_EX_NOPARENTNOTIFY, "BUTTON", "&Add from page with MP3WebIt...",
            WS_VISIBLE | BS_PUSHBUTTON | BS_TEXT | WS_CHILD | WS_TABSTOP,
            buttonRect.left, buttonRect.top,
            (buttonRect.right - buttonRect.left),
            (buttonRect.bottom - buttonRect.top),
            hwndDialog, (HMENU) IDM_WEBIT, NULL, NULL);

        // Hook the dialog proc.
        OldOpenLocationProc = (WNDPROC) SetWindowLong(hwndDialog,
            GWL_WNDPROC, (LONG) OpenLocationProc);
      }
    }
    // allow drop-through to old handler
  }
#endif

  // otherwise allow the original class handler take over.
  if (urldialogtrigger > 0) urldialogtrigger--;
  return CallWindowProc(OldPlayListProc, hwndDlg, uMsg, wParam, lParam);
}

//////////////////////////////////////////////////////////////////////////////

#ifndef WINAMP1       // Winamp 2.x
LRESULT CALLBACK OpenLocationProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
  if (uMsg == WM_COMMAND && HIWORD(wParam) == BN_CLICKED && LOWORD(wParam) == IDM_WEBIT)
  {
    // Winamp 2.x - catch requests to open our dialog
    DialogBox(plugin.hDllInstance, MAKEINTRESOURCE(IDD_DIALOG1),
      hwndDlg, (DLGPROC)ConfigProc);
    return FALSE;
  }

  // otherwise allow the original dialog handler take over.
  return CallWindowProc(OldOpenLocationProc, hwndDlg, uMsg, wParam, lParam);
}
#endif

//////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
  static char inifile[MAX_PATH];
  UNUSED_ARGUMENT(lParam);
  switch (uMsg)
  {
    case WM_INITDIALOG:
    {
      // generate the pathname of the configuration file
      GetModuleFileName(plugin.hDllInstance, inifile, sizeof(inifile));
      char *p = strrchr(inifile, '\\');
      if (p) p++; else p = inifile;
#ifdef WINAMP1
      strcpy(p, "plugin.ini");
#else
      strcpy(p, "winamp.ini");
#endif

      // load the combo box with the previous URLs
      HWND combobox = GetDlgItem(hwndDlg, IDC_COMBOBOX1);
      for (int i = 1; i <= HISTORYCOUNT; i++)
      {
        char keyname[20], buffer[200];
        wsprintf(keyname, "location%i", i);
        GetPrivateProfileString(INISECTION, keyname, "",
            buffer, sizeof(buffer), inifile);
        if (strlen(buffer))
          SendMessage(combobox, CB_ADDSTRING, 0, (LPARAM)buffer);
      }

      // initially check the "clear playlist" box
      SendMessage(GetDlgItem(hwndDlg, IDC_CHECKCLEAR), BM_SETCHECK, 1, 0);

      // set the default text in the edit box to something
      SetDlgItemText(hwndDlg, IDC_COMBOBOX1, "http://");
      SendMessage(GetDlgItem(hwndDlg, IDC_COMBOBOX1), EM_LIMITTEXT, MAX_URL_LENGTH, 0);

      break;
    }
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:
        {
          // retreive what the user typed
          char userinput[MAX_URL_LENGTH];
          GetDlgItemText(hwndDlg, IDC_COMBOBOX1, userinput, MAX_URL_LENGTH);

          // if the "clear playlist" checkbox is checked, then do it
          if (SendMessage(GetDlgItem(hwndDlg, IDC_CHECKCLEAR), BM_GETCHECK, 0, 0))
            SendMessage(plugin.hwndParent, WM_WA_IPC, 0, IPC_DELETE);

          // try to fetch that url
          HCURSOR hourglass = LoadCursor(NULL, IDC_WAIT);
          HCURSOR lastcursor = SetCursor(hourglass);
          bool result = RetrievePlayList(plugin.hwndParent, userinput);
          SetCursor(lastcursor);

          // see if we were successful
          if (result)
          {
            int lastmove = HISTORYCOUNT - 1;
            for (int i = 1; i <= HISTORYCOUNT; i++)
            {
              char keyname[20], buffer[200];
              wsprintf(keyname, "location%i", i);
              GetPrivateProfileString(INISECTION, keyname, "",
                buffer, sizeof(buffer), inifile);
              if (strcmp(buffer, userinput) == 0)
                lastmove = i - 1;
            }
            for (int j = lastmove; j >= 1; j--)
            {
              char keyname[20], buffer[200];
              wsprintf(keyname, "location%i", j);
              GetPrivateProfileString(INISECTION, keyname, "",
                buffer, sizeof(buffer), inifile);
              wsprintf(keyname, "location%i", j + 1);
              WritePrivateProfileString(INISECTION, keyname,
                strlen(buffer) ? buffer : NULL, inifile);
            }
            WritePrivateProfileString(INISECTION, "location1", userinput, inifile);
            EndDialog(hwndDlg, 0);
          }
          break;
        }
        case IDCANCEL:
          EndDialog(hwndDlg,0);
          break;
        case IDHELP:
          AboutBox(plugin.hwndParent);
          break;
    }
  }
  return FALSE;
}

//////////////////////////////////////////////////////////////////////////////


